"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProviderMock = exports.Describer = void 0;
var ShouldBeMockedDependencyError_1 = require("../Errors/ShouldBeMockedDependencyError");
var Describer = /** @class */ (function () {
    function Describer() {
    }
    Object.defineProperty(Describer, "describe", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (val, found) {
            if (found === void 0) { found = {}; }
            if (val === undefined || val == null)
                return [];
            var keys = Object.getOwnPropertyNames(val).concat(Describer.describe(Object.getPrototypeOf(val)));
            if (keys.every(function (e) { return found[e]; }))
                return Object.keys(found);
            var dict = keys.reduce(function (a, b) {
                var _a;
                return (__assign(__assign({}, a), (_a = {}, _a[b] = true, _a)));
            }, found);
            return Describer.describe(val.constructor.prototype, dict);
        }
    });
    return Describer;
}());
exports.Describer = Describer;
exports.ProviderMock = {
    /**
     * Turns provider into a mock-fixture
     * when directly asked to provide it will provide a proper instance
     * Anything the proper instance depend on will be mocked, but still provided
     * Note this will still respect lifetimes (i.e. if a singleton is requested multiple times it will be the same instance)
     * Any function called on dependencies will by default throw an error which you need to override with something like jest.spyOn(..).returns(..)
     * Any other property will be defaulted to undefined
     * Note this is not true for dynamic functions and properties which gets set or changed outside of the dependency injection
     */
    mock: function (provider, setup) {
        var services = provider._.services;
        var template = services.template;
        Object.keys(template).forEach(function (key) {
            var lifetime = services.get(key);
            if (!lifetime)
                return;
            var originalFactory = lifetime.factory;
            lifetime.factory = function (provider) {
                var depth = Object.keys(provider._.validation.trail).length;
                switch (depth) {
                    // Return SUT instance as expected
                    case 1:
                        return originalFactory(provider);
                    // Instantiate dependencies and let the setup function do what it needs
                    // This is required as need to have all expected properties on the object
                    case 2:
                    default:
                        // Instantiate dependencies as we need to have all their properties existing
                        var value_1 = originalFactory(provider);
                        var keys = Describer.describe(value_1);
                        keys.forEach(function (k) {
                            if (
                            // @ts-ignore
                            (value_1.__lookupGetter__ && value_1.__defineGetter__ && value_1.__lookupGetter__(k)) ||
                                // @ts-ignore
                                (value_1.__lookupSetter__ && value_1.__defineSetter__ && value_1.__lookupSetter__(k))) {
                                // @ts-ignore
                                if (value_1.__lookupGetter__ && value_1.__defineGetter__ && value_1.__lookupGetter__(k)) {
                                    // @ts-ignore
                                    value_1.__defineGetter__(k, function () {
                                        throw new ShouldBeMockedDependencyError_1.ShouldBeMockedDependencyError(lifetime.name, k, 'get');
                                    });
                                }
                                // @ts-ignore
                                if (value_1.__lookupSetter__ && value_1.__defineSetter__ && value_1.__lookupSetter__(k)) {
                                    // @ts-ignore
                                    value_1.__defineSetter__(k, function () {
                                        throw new ShouldBeMockedDependencyError_1.ShouldBeMockedDependencyError(lifetime.name, k, 'set');
                                    });
                                }
                                return;
                            }
                            // @ts-ignore
                            if (typeof value_1[k] === 'function') {
                                // @ts-ignore
                                value_1[k] = function () {
                                    throw new ShouldBeMockedDependencyError_1.ShouldBeMockedDependencyError(lifetime.name, k, 'function');
                                };
                                return;
                            }
                        });
                        // Let the user provide whichever mock-overlay desired
                        if (setup && setup[key])
                            setup[key](value_1, provider);
                        return value_1;
                }
            };
        });
        return provider;
    },
};
